//Titel:        JTree
//Version:       1.0
//Copyright:     Copyright (c) 2203
//Autor:         Dipl.-Inf., Dipl.-Ing. (FH) Michael Wilhelm
//Organisation:  HS Harz
//Beschreibung:  Einfhrung in JTree


// Einfaches Beispiel mit DefaultMutableTreeNode
// MyTreeNodeRec liefert Zusatzinformationen, mit toString
// Klick Ereignis


import java.awt.*;
import javax.swing.*;

import java.awt.event.*;

import javax.swing.tree.DefaultMutableTreeNode;

import javax.swing.tree.*;

import javax.swing.event.*;

import java.io.*;

import java.util.Vector;

public class Tree08 extends JFrame {

  JTree jTree1;
  DefaultMutableTreeNode rootNode;
  DefaultTreeModel treeModel;
  TreePath RootPath;

  int _index;


  public Tree08() {
    setGUI();
    this.setLocation(500,300);
  }

  private void setGUI()  {
    setSize(400, 300);
    setTitle("JTree 8. Beispiel");
    this.getContentPane().setLayout( new BorderLayout() );


    //---------------------------------------
    rootNode = BuildTree();

    treeModel = new DefaultTreeModel(rootNode);
    jTree1 = new javax.swing.JTree(treeModel);  //  kein Root !!!, hier ein Defaultmodell

    //aktuellen Pfad:
    //  RootPath = jTree1.getSelectionPath();

    // Alle Knoten lschen:
    // rootNode.removeAllChildren();


    // Neu
    treeModel.addTreeModelListener(new MyTreeModelListener());



    jTree1.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    // jTree1.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
   //  SINGLE_TREE_SELECTION, CONTIGUOUS_TREE_SELECTION or DISCONTIGUOUS_TREE_SELECTION.



     // aus TreeDemo
    jTree1.addTreeSelectionListener(  new MyTreeSelectionListener(jTree1) );


     MouseListener ml = new MyMouseAdapter(jTree1);
     jTree1.addMouseListener(ml);


    jTree1.setBorder(BorderFactory.createLineBorder(Color.black));
    jTree1.setToolTipText("ToolTipp");
     // zeigt eine Linie zur Wurzel
    jTree1.setShowsRootHandles(true);

      // Horizontale Linien
    jTree1.putClientProperty("JTree.lineStyle","Horizontal");
      // Mit Linien zwischen den Knoten
    jTree1.putClientProperty("JTree.lineStyle","Angled");

    this.getContentPane().add(jTree1, BorderLayout.CENTER);

    // Neue Version
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

  }  // setGUI






  // ----------------------------
  DefaultMutableTreeNode BuildTree() {
    DefaultMutableTreeNode root;
    DefaultMutableTreeNode[]  Knoten = new DefaultMutableTreeNode[3];

    root = createMyNode(  0,"root");  // new DefaultMutableTreeNode("root");

      // liefert einen DefaultMutableTreeNode
    Knoten[0] = createMyNode(  1,"1. Stufe (1)");
    root.add( Knoten[0] );

    Knoten[1] = createMyNode(  2,"AAA");
    Knoten[0].add( Knoten[1] );  // Parameter Vater
    Knoten[1] = createMyNode(  3,"BBB");
    Knoten[0].add( Knoten[1] );  // Parameter Vater


    // ---------

    Knoten[0] = createMyNode(  2,"1. Stufe (2)");
    root.add( Knoten[0] );

    Knoten[1] = createMyNode(  6,"CCC");
    Knoten[0].add( Knoten[1] );  // Parameter Vater
    Knoten[1] = createMyNode(  2,"DDDD");
    Knoten[0].add( Knoten[1] );  // Parameter Vater


    Knoten[0] = createMyNode(  3,"1. Stufe (3)");
    root.add( Knoten[0] );

    Knoten[1] = createMyNode(  6,"EEE");
    Knoten[0].add( Knoten[1] );  // Parameter Vater

      Knoten[2] = createMyNode(  7,"E1E1E1");
      Knoten[1].add( Knoten[2] );  // Parameter Vater
      Knoten[2] = createMyNode(  2,"E2E2E2");
      Knoten[1].add( Knoten[2] );  // Parameter Vater

    Knoten[1] = createMyNode(  11,"FFF");
    Knoten[0].add( Knoten[1] );  // Parameter Vater
      Knoten[2] = createMyNode(  2,"F1F1F1");
      Knoten[1].add( Knoten[2] );  // Parameter Vater
      Knoten[2] = createMyNode(  4,"F2F2F2");
      Knoten[1].add( Knoten[2] );  // Parameter Vater

    return root;
  }  // BuildTree



 DefaultMutableTreeNode createMyNode(int myData,  String nodeString) {
    MyTreeNodeRec myNode;
    DefaultMutableTreeNode Node;

    myNode = new  MyTreeNodeRec(myData, nodeString);
    Node = new DefaultMutableTreeNode(myNode);
    // Node = new DefaultMutableTreeNode(nodeString);
    //Node.setUserObject(myNode);
    return Node;
 }


 
  //berschreiben, damit das Programm bei Herunterfahren des Systems beendet werden kann
  /*
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if(e.getID() == WindowEvent.WINDOW_CLOSING) {
      System.exit(0);
    }
  }
  */
  
  public static void main(String[] args) {
    Tree08 frame = new Tree08();
    frame.setVisible(true);
 }


}  // JTree8





// Einfache Klasse fr jeden Knoten
class MyTreeNodeRec  {
  public static long nr=0;

    private long _id = 0; // Id des Knotens  z.B. als ID

    private int _myData;

    public String _nodeString;


   public MyTreeNodeRec(int myData,  String nodeString) {
      _myData = myData;
      _nodeString = nodeString;  // fr JTree;
      nr++;
      _id = nr;  // setzt eindeutige Nummer
   }


   public long getID() {
     return _id;
   }


   public int getData() {
     return _myData;
   }

   public void setData(int myData) {
     _myData = myData;
   }
   
   // sehr wichtig
   public String toString() {
     return _nodeString;
   }
   

}  // MyTreeNodeRec


// -------------------------------------------------------------
// -------------------------------------------------------------

// Erkennt Single-Click und Double-Klick
class MyMouseAdapter extends MouseAdapter {
   private JTree jTree1;

  public MyMouseAdapter(JTree jTree1) {
     this.jTree1 = jTree1;
  }

   public void mouseClicked(MouseEvent e) {
      int selRow = jTree1.getRowForLocation(e.getX(), e.getY());
      TreePath selPath = jTree1.getPathForLocation(e.getX(), e.getY());
      if(selRow != -1) {
         if(e.getClickCount() == 1) {
             mySingleClick(selRow, selPath);
         }
         else if(e.getClickCount() == 2) {
             myDoubleClick(selRow, selPath);
         }
     }
  }

  void  mySingleClick(int selRow, TreePath selPath) {
    System.out.println("mySingleClick: "+selRow);
  }

  void  myDoubleClick(int selRow, TreePath selPath) {
    System.out.println("myDoubleClick: "+selRow);
  }

}  // MyMouseAdapter




// -------------------------------------------------------------
// -------------------------------------------------------------
class MyTreeSelectionListener implements TreeSelectionListener {
   private JTree jTree1;

   private MyTreeSelectionListener() {
   }


   public MyTreeSelectionListener(JTree jTree1) {
     this.jTree1 = jTree1;
   }

      public void valueChanged(TreeSelectionEvent e) {
          MyTreeNodeRec myNode;
          int data;
          DefaultMutableTreeNode node;
          // e.getSource()
          node = (DefaultMutableTreeNode) jTree1.getLastSelectedPathComponent();
          if (node == null) return;
          myNode = (MyTreeNodeRec) node.getUserObject();

          System.out.println( "MyTreeSelectionListener: "+jTree1.getLastSelectedPathComponent() );

          data = myNode.getData();
          if (data!=0) {
              /**
                 Node is clicked, so send a message to the desktop
                 to activate the frame or create a new frame
               */
              // _desktop.activateItemInWorkplace(myNode);
          }
      }


}



class MyTreeModelListener implements TreeModelListener {

    public void treeNodesChanged(TreeModelEvent e) {
        DefaultMutableTreeNode node;
        node = (DefaultMutableTreeNode)
                 (e.getTreePath().getLastPathComponent());

        /*
         * If the event lists children, then the changed
         * node is the child of the node we've already
         * gotten.  Otherwise, the changed node and the
         * specified node are the same.
         */
        try {
            int index = e.getChildIndices()[0];
            node = (DefaultMutableTreeNode)
                   (node.getChildAt(index));
        } catch (NullPointerException exc) {}

        System.out.println("The user has finished editing the node.");
        System.out.println("New value: " + node.getUserObject());
    }

    public void treeNodesInserted(TreeModelEvent e) {
      System.out.println("MyTreeModelListener: treeNodesInserted");
    }

    public void treeNodesRemoved(TreeModelEvent e) {
      System.out.println("MyTreeModelListener: treeNodesRemoved");
    }

    public void treeStructureChanged(TreeModelEvent e) {
      System.out.println( "MyTreeModelListener: treeStructureChanged");
    }
}

